import toml
from more_itertools import flatten
from jaraco.context import suppress
from jaraco.functools import apply


def none_as_empty(ob):
    """
    >>> none_as_empty({})
    {}
    >>> none_as_empty(None)
    {}
    >>> none_as_empty({'a': 1})
    {'a': 1}
    """
    return ob or {}


@apply(none_as_empty)
@suppress(Exception)
def read_opts(filename):
    with open(filename) as strm:
        defn = toml.load(strm)
    return defn["tool"]["jaraco"]["pytest"]["opts"]


def get_supported_options(parser):
    """
    >>> from jaraco.test.pytest.test_allow import Bunch
    >>> parser = Bunch(
    ...     _groups = [Bunch(
    ...         options = [Bunch(
    ...             _long_opts=['--opt'])])])
    >>> list(get_supported_options(parser))
    ['--opt']
    """
    return flatten(
        opts._long_opts for group in parser._groups for opts in group.options
    )


def pytest_addoption(parser, pluginmanager):
    opts = read_opts('pyproject.toml')
    missing = set(opts) - set(get_supported_options(parser))
    for opt in missing:
        parser.addoption(opt, **opts[opt])



import textwrap
import pathlib
import unittest.mock

import pytest
import jaraco.collections
from jaraco.test.pytest import allow


class Bunch(dict, jaraco.collections.ItemsAsAttributes):
    pass


@pytest.fixture
def tmpdir_cur(tmpdir):
    with tmpdir.as_cwd():
        yield tmpdir


def test_pytest_addoption(tmpdir_cur):
    parser = Bunch(_groups=[])
    parser.addoption = unittest.mock.MagicMock()
    pathlib.Path('pyproject.toml').write_text(
        textwrap.dedent(
            """
            [tool.jaraco.pytest.opts.--opt]
            bar = "baz"
            """
        )
    )
    allow.pytest_addoption(parser, None)
    parser.addoption.assert_called_with('--opt', bar="baz")



import textwrap
import pathlib
from unittest import mock

import pytest
import jaraco.collections
from jaraco.test.pytest import enabler


class Bunch(dict, jaraco.collections.ItemsAsAttributes):
    pass


@pytest.fixture
def tmpdir_cur(tmpdir):
    with tmpdir.as_cwd():
        yield tmpdir


def test_pytest_addoption(tmpdir_cur):
    pathlib.Path('pyproject.toml').write_text(
        textwrap.dedent(
            """
            [tool.jaraco.pytest.plugins.black]
            addopts = "--black"
            """
        )
    )
    config = mock.MagicMock()
    config.pluginmanager.has_plugin = lambda name: name == 'black'
    args = []
    enabler.pytest_load_initial_conftests(config, None, args)
    assert args == ['--black']



import shlex

import toml
from jaraco.context import suppress
from jaraco.functools import apply


def none_as_empty(ob):
    """
    >>> none_as_empty({})
    {}
    >>> none_as_empty(None)
    {}
    >>> none_as_empty({'a': 1})
    {'a': 1}
    """
    return ob or {}


@apply(none_as_empty)
@suppress(Exception)
def read_plugins(filename):
    with open(filename) as strm:
        defn = toml.load(strm)
    return defn["tool"]["jaraco"]["pytest"]["plugins"]


def pytest_load_initial_conftests(early_config, parser, args):
    plugins = read_plugins('pyproject.toml')
    matches = filter(early_config.pluginmanager.has_plugin, plugins)
    for match in matches:
        args.extend(shlex.split(plugins[match].get('addopts', "")))
